REM Generated by Genezzo::Havok::SysHook version 7.04 on 2006-02-25T23:05:30
REM
REM Copyright (c) 2005 Jeffrey I Cohen.  All rights reserved.
REM
REM 
ct sys_hook xid=n pkg=c hook=c replace=c xtype=c xname=c args=c owner=c creationdate=c version=c
i havok 3 Genezzo::Havok::SysHook SYSTEM 2006-02-25T23:05:30 0 7.04

REM HAVOK_EXAMPLE
i sys_hook 1 Genezzo::Dict dicthook1 Howdy_Hook require Genezzo::Havok::Examples Howdy SYSTEM 2006-02-25T23:05:30 0
i sys_hook 2 Genezzo::Dict dicthook1 Ciao_Hook  require Genezzo::Havok::Examples Ciao SYSTEM 2006-02-25T23:05:30 0



commit
shutdown
startup
